/*
 * Copyright 2024 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

#pragma once
#include <string>

struct StatusErrorInfo {
    int32_t errorCode;
    std::string errorMessage;
};
class KnowledgebaseClient {
public:
    static KnowledgebaseClient* getInstance();
    ~KnowledgebaseClient();

    bool isSupported();

    /***   @brief create knowledge base
     *   @param [in] input, 当前入参中的knowledgebaseName中的值设为'default'即可；
     *   such as:
     *   {
     *     "knowledgebase_name": "default"
     *   }
     *   @param [out] result. 返回的knowledgebasename。在后面的接口中需要用这个名字
     *   @param [return] 错误状态信息，是一个结构体，形式如下：
     *     struct StatusErrorInfo{
     *         std::string errorCode;
     *         std::string errorMessage;
     *      };
     ***/
    StatusErrorInfo createKnowledgebase(const std::string& input, std::string& result);

    /***
     *   @brief delete knowledge base
     *   @param [in] input, knowledgebase的名字，输入为createknowledgebase返回的knowledgebaseName字段，是一个json格式
     *    such as :
     *     {
     *        "knowledgebase_name": "test"
     *     }
     *   @param [return] 错误状态信息，是一个结构体，形式如下：
     *     struct StatusErrorInfo{
     *         std::string errorCode;
     *         std::string errorMessage;
     *      };
     ***/
    StatusErrorInfo deleteKnowledgebase(const std::string& input);

    /***
     *    @brief similarity search
          @param [in] content of the search and the name of the knowledge base, a json string.
          such as:
           {
            "knowledgebase_name": "test",
            "search_condition": "linux内核安全选项"
           }
          @param [out] result, a json.
          such as:  {
                      [
                        {
                            "document": "adgjcvn",
                            "filepath": "/path/to/file1.txt",
                            "similarity": 0.92
                        },
                        {
                            "document": "trsghncd",
                            "filepath": "/path/to/file2.txt",
                            "similarity": 0.85
                        }
                        ......
                      ]
                    }
    *   @param [return] 错误状态信息，是一个结构体，形式如下：
    *     struct StatusErrorInfo{
    *         std::string errorCode;
    *         std::string errorMessage;
    *      };
     * ***/
    StatusErrorInfo similaritySearch(const std::string& input, std::string& result);

    /***
         @brief add text file to user data
          @param [in] input files info, a json. chunk_size是文本分块的大小；
          such as: {
                      [
                        {
                          "knowledgebase_name": "test",
                          "filepath": "/path/to/file1.txt",
                          "fileformat": "txt"
                          "chunk_size": 800 (底层写死为800)
                        },
                        {
                          "knowledgebase_name": "test",
                          "filepath": "/path/to/file2.txt",
                          "fileformat": "txt"
                          "chunk_size": 800 (底层写死为800)
                        }
                      ]
                    }
    *      @param [return] 错误状态信息，是一个结构体，形式如下：
    *     struct StatusErrorInfo{
    *         std::string errorCode;
    *         std::string errorMessage;
    *      };
     * ***/
    StatusErrorInfo addTextFiles(const std::string& input);
    /***
          @brief get text file contents to user data
          @param [in] input files info, a json.
          such as:  {
                      [
                        {
                          "knowledgebase_name": "test",
                          "filepath": "/path/to/file1.txt",
                          "fileformat": "txt"
                          "chunk_size": 100
                        },
                        {
                          "knowledgebase_name": "test",
                          "filepath": "/path/to/file2.txt",
                          "fileformat": "txt"
                          "chunk_size": 200
                        }
                      ]
                    }
          @param [out] result info contents, a json ;
          such as:  {
                        [
                            {
                            "filepath": "/path/to/file1.txt",
                            "document": "file1 content"
                            },
                            {
                            "filepath": "/path/to/file2.txt",
                            "document": "file2 content"
                            }
                        ]
                    }
    *      @param [return] 错误状态信息，是一个结构体，形式如下：
    *     struct StatusErrorInfo{
    *         std::string errorCode;
    *         std::string errorMessage;
    *      };
     * ***/
    StatusErrorInfo getTextFileContents(const std::string& input, std::string& result);

private:
    KnowledgebaseClient();
    void init();
    void destroy();
    StatusErrorInfo ParserResultsErrorInfo(const std::string& result);
    std::string ParserResultsData(const std::string& result);
    std::string ParserKnowledgetbaseName(const std::string& result);

private:
    const char* objectPath_ = "/com/kylin/AiBusiness/Knowledgebase";
};